/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>



#define VER     "0.1"
#define B64DECX bb = *bf; \
                for(; \
                bb && \
                ((bb == ' ') || \
                (bb == '\t') || \
                (bb == '\r') || \
                (bb == '\n')); \
                bf++);



typedef unsigned char   u_char;



void chkp3_pwd(u_char *pwd);
void Base64Decode(u_char *buf, u_char *ret);



int main(int argc, char *argv[]) {
    setbuf(stdout, NULL);

    fputs("\n"
        "CheckP3 password decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\nUsage: %s <password>\n"
            "\n"
            "Passwords are stored in the registry HKEY_CURRENT_USER\\Software\\ChkP3\\ACCOUNT\n"
            "where ACCOUNT is the specific account that contains the password\n"
            "\n", argv[0]);
        exit(1);
    }

    chkp3_pwd(argv[1]);
    printf("\nPassword:   %s\n\n", argv[1]);
    return(0);
}



void chkp3_pwd(u_char *pwd) {
    u_char  *out,
            *ptr,
            dl,
            ebp9;
    int     len,
            i;
    static  u_char  encdata[] =
        "\x03\x99\x13\xA6\x29\xF7\x50\x5E\xA4\x65\x2F\x02\x64\xC5\xFB\xD4"
        "\x9B\x80\x8C\x9D\xF1\x1F\xB8\x0E\x9A\xA5\x06\xA3\x21\x4D\xCA\x31"
        "\x91\x73\xD2\x2B\x08\x19\x77\x0F\x52\xEE\x71\x70\x63\x39\x44\x60"
        "\x01\x5F\xB9\x96\xA0\x24\x2C\xAD\x18\x11\x82\x7A\x0A\xE5\x4E\xD9"
        "\x81\x42\x89\x5D\xC4\x87\x3F\x75\x3C\x68\xE8\x33\xE9\xEB\x9C\x3B"
        "\xE7\x22\x55\x4A\x7C\x46\x8A\xE3\x30\xDD\xB3\xA1\x35\xCC\xC3\x27"
        "\x92\x25\xF0\x43\xED\x6F\x4B\x97\x6B\xA7\x54\x51\x16\xB1\x72\x6E"
        "\x8D\xD0\x85\x38\xBB\x41\xAA\x78\x79\xDC\x1B\x4C\x94\xEA\xEC\x61"
        "\xAC\x93\x9F\x62\xAB\x0C\x7D\xD6\xBA\x66\xB2\xCF\xF9\xF2\xC0\xCD"
        "\xB0\xFC\xE2\xF5\x5B\x0D\x07\x6A\xB5\xF8\x53\x36\x0B\x14\x20\x76"
        "\x6C\x98\xDE\xFE\xE6\x58\xD8\xEF\x05\x88\xBD\x5A\x47\x74\xDB\x15"
        "\x28\x4F\x6D\x37\x1E\xC7\xAF\xD7\x7F\xDF\x8B\xFD\x59\xC6\x2E\xF3"
        "\x90\xD5\x56\x1D\x67\x49\x69\x48\x8E\xF6\xC9\xBE\x83\xD3\xA9\xA2"
        "\x84\x45\x04\x17\x3D\xBC\xB7\xFA\x12\xB6\x34\x26\x3E\x7E\xE1\x2A"
        "\x2D\xAE\xDA\x1A\xA8\x7B\x9E\xBF\xE0\xCB\xFF\xCE\xC1\x57\x10\xB4"
        "\xF4\x1C\x32\x95\x8F\xE4\x09\x86\xC2\xD1\xC8\x5C\x40\x00\x3A\x23";

    Base64Decode(pwd, pwd);

    len = strlen(pwd) >> 1;
    out = ptr = pwd;
    for(i = 0; i < len; i++) {
        dl = pwd[1];
        switch(pwd[0]) {
            case 0x4d: dl -= 0x20; break;
            case 0x54: dl += 0x60; break;
            case 0x5a: dl += 0x96; break;
        }
        *out++ = dl;
        pwd += 2;
    }

    dl = ebp9 = 0;
    for(i = 1; i <= len; i++) {
        ebp9 = encdata[i];
        dl += ebp9;
        encdata[i] = encdata[dl];
        encdata[dl] = ebp9;
        ebp9 += encdata[i];
        *ptr ^= encdata[ebp9];
        ptr++;
    }
    *ptr = 0;
}



void Base64Decode(u_char *buf, u_char *ret) {
    int     c,
            d,
            i,
            *Base;
    u_char  bb,
            *bf,
            *rf;

    if(!*buf) {
        *ret = 0;
        return;
    }

    Base = (int *)malloc(sizeof(int) << 8);
    if(!Base) {
        strcpy(ret, buf);
        return;
    }
    for(i = 0; i < 26; i++) {
        Base['A' + i] = i;
        Base['a' + i] = i + 26;
    }
    for(i = 0; i < 10; i++) {
        Base['0' + i] = i + 52;
    }
    Base['+'] = 62;
    Base['/'] = 63;

    bf = buf;
    rf = ret;
    B64DECX;
    while(1) {
        bb = *bf++;
        if((bb == '=') || !bb) break;
        c = Base[bb];

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        d = Base[bb];
        *rf++ = (c << 2) | (d >> 4);

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        c = Base[bb];
        *rf++ = ((d & 0xF) << 4) | (c >> 2);

        B64DECX;
        bb = *bf++;
        if((bb == '=') || !bb) break;
        *rf++ = ((c & 0x3) << 6) | Base[bb];
    }
    *rf = 0;
    free(Base);
}



